/*
 * Author : Gouta Ilyes
 * Subject : Win32 GUI code / igouta@voila.fr
 * Notes :
 *
 */

#include <windows.h>
#include <commctrl.h>
#include <dsound.h>
#include "resource.h"

#include "audio.h"
#include "dispatch.h"
#include "gui.h"

#define DB      {                                              \
				char b[256];                                   \
				wsprintf(b,"Error code : %d",GetLastError());  \
				MessageBox(hWnd,b,App,MB_ICONSTOP);            \
				}                                              \

#define WM_EOF           WM_USER+1

extern char App[];

static HWND hWnd;
static HANDLE hPlayer;
static DWORD tID;

static char fileName[MAX_PATH];
static char fullName[MAX_PATH];
static int playlistMode;

static char notice[]="amp v0.7.6 port on Win32 by Gouta Ilyes 2002.\r\nOriginal code (C) Tomislav Uzelac 1996-1997, modified since by many other people.";

BOOL CALLBACK dlgProc(HWND hWnd,UINT Message,WPARAM wParam,LPARAM lParam) {
	switch(Message) {
	case WM_INITDIALOG: {
		SendMessage(GetDlgItem(hWnd,ID_SLIDER),TBM_SETRANGE,TRUE,(LPARAM) MAKELONG(0,255));
		SendMessage(GetDlgItem(hWnd,ID_SLIDER),TBM_SETPOS,TRUE,(LPARAM) audioGetVolume());
		return TRUE;
						}
	case WM_HSCROLL:
		audioSetVolume(SendMessage(GetDlgItem(hWnd,ID_SLIDER),TBM_GETPOS,0,0));
		return 0;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDCANCEL:
			EndDialog(hWnd,TRUE);
			return TRUE;
		default: return FALSE;
		}
	default: return FALSE;
	}
}

BOOL CALLBACK aboutProc(HWND hWnd,UINT Message,WPARAM wParam,LPARAM lParam) {
	switch(Message) {
	case WM_INITDIALOG: {
		HRSRC res;HGLOBAL resource;char* data;
		res=FindResource(NULL,MAKEINTRESOURCE(ID_ORGLICENSE),"TEXT");
		if (!res) return TRUE;
		resource=LoadResource(NULL,res);
		data=(char*) LockResource(resource);
		SendDlgItemMessage(hWnd,ID_MINE,WM_SETTEXT,0,(LPARAM) notice);
		SendDlgItemMessage(hWnd,ID_ORIGINAL,WM_SETTEXT,0,(LPARAM) data);
		return TRUE;
						}
	case WM_COMMAND:
	switch (LOWORD(wParam)) {
	case IDCANCEL:
		EndDialog(hWnd,TRUE);
		return TRUE;
	default: return FALSE;
	}
	default: return FALSE;
	}
}

DWORD WINAPI playerThread(LPVOID s) {
	if (!fileName[0]) return 1;
	in_file=CreateFile(fileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if (in_file==INVALID_HANDLE_VALUE) return 1;

	EnableMenuItem(GetMenu(hWnd),ID_PLAY,MF_BYCOMMAND|MF_GRAYED);
	EnableMenuItem(GetMenu(hWnd),ID_STOP,MF_BYCOMMAND|MF_ENABLED);

	decodeMPEG();

	EnableMenuItem(GetMenu(hWnd),ID_PLAY,MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd),ID_STOP,MF_BYCOMMAND|MF_GRAYED);
	
	close_audio();
	CloseHandle(in_file);

	PostMessage(hWnd,WM_EOF,0,0); // we're done
	return 0;
}

static void gui_open(HWND hWnd) {
	OPENFILENAME open;
	playlistMode=0;
	ZeroMemory(&open,sizeof(open));
	open.lStructSize=sizeof(open);
	open.hwndOwner=hWnd;
	open.hInstance=GetModuleHandle(NULL);
	open.lpstrFile=fileName;
	open.nMaxFile=sizeof(fileName);
	open.lpstrFilter="MPEG audio files (*.mp2,*.mp3)\0*.mp2;*.mp3\0";
	open.lpstrInitialDir=NULL;
	open.Flags=OFN_FILEMUSTEXIST|OFN_NONETWORKBUTTON|OFN_HIDEREADONLY;
	if (!GetOpenFileName(&open)) ZeroMemory(fileName,sizeof(fileName));
}

void process_next() {
	static int currentSong=0;
	char buffer[MAX_PATH];
	char path[MAX_PATH];char *p;int i=0;
	FILE *playlist;
	
	if (!playlistMode) return;

	playlist=fopen(fullName,"rt");
	if (!playlist) return;

	fgets(buffer,sizeof(buffer),playlist); // skip the two first lines
	fgets(buffer,sizeof(buffer),playlist);

	while (i<=currentSong) {
		fgets(buffer,sizeof(buffer),playlist);
		for (p=buffer+sizeof(buffer);p>=buffer;p--) if (*p=='\n'||*p=='\r') *p=0;
		p=fullName+lstrlen(fullName);
		while (*p!='\\'&&p>=fullName) p--;
		ZeroMemory(fileName,sizeof(fileName));
		lstrcpyn(fileName,fullName,++p-fullName+1);
		lstrcat(fileName,buffer);
		if (!fgets(buffer,sizeof(buffer),playlist)) { // skip the two first lines
			if (feof(playlist)) {currentSong=0;break;}
		}
		i++;
	}
	
	currentSong++;
	fclose(playlist);

	gui_play();
}

void gui_playlist() {
	OPENFILENAME open;
	playlistMode=0;
	ZeroMemory(&open,sizeof(open));
	open.lStructSize=sizeof(open);
	open.hwndOwner=hWnd;
	open.hInstance=GetModuleHandle(NULL);
	open.lpstrFile=fullName;
	open.nMaxFile=sizeof(fullName);
	open.lpstrFilter="m3u playlist (*.m3u)\0*.m3u\0";
	open.lpstrInitialDir=NULL;
	open.Flags=OFN_FILEMUSTEXIST|OFN_NONETWORKBUTTON|OFN_HIDEREADONLY;
	if (!GetOpenFileName(&open)) ZeroMemory(fullName,sizeof(fullName));
	playlistMode=1;
	process_next();
}

void gui_about(HWND hWnd) {
	DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(ID_ABOUTDLG),hWnd,aboutProc);
}

void gui_play() {
	hPlayer=CreateThread(NULL,0,(LPTHREAD_START_ROUTINE) playerThread,NULL,0,&tID);
}

void gui_stop() {
	if (hPlayer) {
		stopMPEG();
		if (WaitForSingleObject(hPlayer,500)==WAIT_TIMEOUT) {
			MessageBox(hWnd,"Failed asking thread to die !",App,MB_ICONSTOP);
			TerminateThread(hPlayer,0);
		}
		close_audio();
		if (in_file) {CloseHandle(in_file);in_file=NULL;}
		CloseHandle(hPlayer);
		hPlayer=NULL;
	}

	EnableMenuItem(GetMenu(hWnd),ID_PLAY,MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd),ID_STOP,MF_BYCOMMAND|MF_GRAYED);
}

int onDestroy(WPARAM wParam,LPARAM lParam) {
	gui_stop();
	PostQuitMessage(0);
	return 0;
}

int onEof(WPARAM wParam,LPARAM lParam) {
	gui_stop();
	if (playlistMode) process_next();
	return 0;
}

int onExit(WPARAM wParam,LPARAM lParam) {
	DestroyWindow(hWnd);
	return 0;
}

int onOpen(WPARAM wParam,LPARAM lParam) {
	gui_open(hWnd);
	return 0;
}

int onPlaylist(WPARAM wParam,LPARAM lParam) {
	gui_playlist();
	return 0;
}

int onAbout(WPARAM wParam,LPARAM lParam) {
	gui_about(hWnd);
	return 0;
}

int onPlay(WPARAM wParam,LPARAM lParam) {
	gui_play();
	return 0;
}

int onStop(WPARAM wParam,LPARAM lParam) {
	playlistMode=0;
	gui_stop();
	return 0;
}

int onVolume(WPARAM wParam,LPARAM lParam) {
	DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(ID_VOLUMEDLG),hWnd,dlgProc);
	return 0;
}

int onCommand(WPARAM wParam,LPARAM lParam) {
	START_DISPATCH(LOWORD(wParam));
	DISPATCH(ID_EXIT,onExit);
	DISPATCH(ID_OPEN,onOpen);
	DISPATCH(ID_PLAYLIST,onPlaylist);
	DISPATCH(ID_ABOUT,onAbout);
	DISPATCH(ID_PLAY,onPlay);
	DISPATCH(ID_STOP,onStop);
	DISPATCH(ID_VOLUME,onVolume);
	END_COM_DISPATCH;
}

LRESULT CALLBACK WndProc(HWND hWnd,UINT Message,WPARAM wParam,LPARAM lParam) {
	START_DISPATCH(Message);
	DISPATCH(WM_DESTROY,onDestroy);
	DISPATCH(WM_EOF,onEof);
	DISPATCH(WM_COMMAND,onCommand);
	END_DISPATCH;
}

int init_gui() {
	WNDCLASS wndClass;

	InitCommonControls();

	wndClass.style=CS_SAVEBITS|CS_BYTEALIGNCLIENT|CS_BYTEALIGNWINDOW|CS_HREDRAW|CS_VREDRAW;
	wndClass.lpfnWndProc=WndProc;
	wndClass.cbClsExtra=0;
	wndClass.cbWndExtra=0;
	wndClass.hInstance=GetModuleHandle(NULL);
	wndClass.hIcon=NULL;
	wndClass.hCursor=LoadCursor(NULL,IDC_ARROW);
	wndClass.hbrBackground = (HBRUSH) (COLOR_BTNFACE+1);
	wndClass.lpszMenuName=MAKEINTRESOURCE(ID_MENU);
	wndClass.lpszClassName=App;

	if (!RegisterClass(&wndClass)) return 0;
	hWnd=CreateWindow(App,App,WS_SYSMENU|WS_CAPTION|WS_MINIMIZEBOX,CW_USEDEFAULT,CW_USEDEFAULT,220,GetSystemMetrics(SM_CYMENU)+GetSystemMetrics(SM_CYCAPTION)+4*GetSystemMetrics(SM_CYBORDER),NULL,NULL,GetModuleHandle(NULL),NULL);
	if (!hWnd) return 0;
	ShowWindow(hWnd,SW_NORMAL);

	EnableMenuItem(GetMenu(hWnd),ID_PLAY,MF_BYCOMMAND|MF_ENABLED);
	EnableMenuItem(GetMenu(hWnd),ID_STOP,MF_BYCOMMAND|MF_GRAYED);

	playlistMode=0;
	return 1;
}
